class BWT(object):
    def __init__(self, text):
        t = text + "$"
        self.position_list = [ i for i in range(len(t)) ]
        self.position_list = sorted(self.position_list, key = lambda s: t[s:])
        self.bwt = "".join( [ t[i-1] for i in self.position_list ] )
        self.ranks = {}
        self.cnums = {}
        for c in self.bwt:
            if not c in self.cnums.keys():
                self.cnums[c] = 0
                self.ranks[c] = []
        for c in self.bwt:
            self.cnums[c] = self.cnums[c] + 1
            for e in self.ranks.keys():
                self.ranks[e].append(self.cnums[e] - 1)
    def search(self, pattern):
        if len(pattern) >= len(self.bwt):
            return []
        i = len(pattern) - 1
        c = pattern[i]
        if not c in self.cnums.keys():
            return []
        first = 0
        for e in self.cnums.keys():
            if e < c:
                first = first + self.cnums[e]
        last = first + self.cnums[c] -1
        while not first > last:
            if i == 0:
                return sorted(self.position_list[first:last+1])
            i = i - 1
            c = pattern[i]
            if not c in self.cnums.keys():
                return []
            first = self.ranks[c][first - 1] + 1
            last = self.ranks[c][last]
            for e in self.cnums.keys():
                if e < c:
                    first = first + self.cnums[e]
                    last = last + self.cnums[e]
        return []
    def get_text(self):
        return "".join([ e[1] for e in sorted( zip(self.position_list, self.bwt) ) ])[1:]
    def from_fasta_file(filename):
        text = ""
        first_line = True
        with open(filename, "r") as f:
            for line in f:
                if first_line:
                    seq_name = line.strip()
                    first_line = False
                else:
                    text = text + line.strip().upper()
        return BWT(text)

if __name__ == "__main__":
    text = "AGATA"
    pattern = "AT"
    bwt = BWT(text)
    print("Burrows Wheeler transform", bwt.bwt)
    print("search found", pattern)
    print("at", bwt.search(pattern))
    print("in", bwt.get_text())
    print("and A at", bwt.search("A"))
    



from distutils.core import setup

long_description = """
**Burrows Wheeler Transform (BWT) search**

Import the `BWT` class into your namespace for convenience:

```
from BWTsearch import BWT
```

Initialize a BWT of text with instantiating the BWT class:

```
text = "AGATA"
```

```
bwt = BWT(text)
```

The instance bwt now will contain all necessary indices for efficiently searching:

- `bwt.bwt` ... contains the BWT string
- `bwt.position_list` ... contains the start positions of the rotations for each character of bwt
- `bwt.ranks` ... contains rank indices for each character of the alphabet in bwt
- `bwt.cnums` ... contains number of character occurances of the alphabet in bwt

Use the `search()` function to obtain a list of positions of exact matches for pattern in text:

```
pattern = "AT"
```

```
bwt.search(pattern)
```

This returns a list with indexes into text where an exact match occurs. Empty list if none.
The pattern will be searched case-sensitive and should not contain $.

Use the `get_text()` function to retrieve the original text from the BWT instance:

```
bwt.get_text()
```

If a sequence file is available, a genome can be loaded from a FASTA file:

```
phix = BWT.from_fast_file("phix.fa")
```

```
phix.search("AGATA")
```
"""

setup(
  name = 'BWTsearch',         # How you named your package folder (MyLib)
  packages = ['BWTsearch'],   # Chose the same as "name"
  version = '0.1.15',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Burrows Wheeler Transform (BWT) search',   # Give a short description about your library
  long_description=long_description,
  long_description_content_type="text/markdown",
)
